import uuid
import urllib.parse
from .File import File
from .Preset import Preset
from .SupplyChain import SupplyChain

class Asset:

    def __init__(self,Rally,id=None,name=None):
        self.Rally = Rally
        if not id and not name:
            self.name = "Int_Test_Asset_{}".format(str(uuid.uuid4()))
            self.id = Rally.apiCall("POST","/assets",body={"data": {"attributes": {"name": self.name }, "type": "assets" }})["data"]["id"]
        elif not id:
            self.name = name
            try:
                self.id = Rally.apiCall("GET","/assets?search=name={}".format(urllib.parse.quote_plus(name)))["data"][0]["id"]
            except IndexError:
                raise ValueError("Could not find asset with name '{}'".format(name)) from None
        else:
            self.id = id
            self.name = name

    def getName(self):
        if not self.name:
            self.name = self.Rally.apiCall("GET","/assets/{}".format(self.id))["data"]["attributes"]["name"]
        return self.name

    def getMetadata(self):
        self.metadata = self.Rally.apiCall("GET","/movies/{}/metadata/Metadata".format(self.id))["data"]["attributes"]["metadata"]
        return self.metadata

    def getWorkflowMetadata(self):
        self.workflowMetadata = self.Rally.apiCall("GET","/movies/{}/metadata/Workflow".format(self.id))["data"]["attributes"]["metadata"]
        return self.workflowMetadata
    
    def delete(self):
        return self.Rally.apiCall("DELETE","/assets/{}".format(self.id),fullResponse=True)

    def preset(self,id=None,name=None):
        return Preset(self,id=id,name=name)

    def supplychain(self,id=None,name=None):
        return SupplyChain(self,id=id,name=name)

    def listFiles(self):
        fileData = self.Rally.apiCall("GET","/assets/{}/files".format(self.id),paginate=True)["data"]
        return [File(self,id=item["id"],label=item["attributes"]["label"]) for item in fileData]

    def getFile(self,id=None,label=None):
        return File(self,id=id,label=label)


from time import sleep,time
import urllib.parse
import json

class SupplyChain:

    def __init__(self,Asset,id=None,name=None):
        if not id and not name:
            raise TypeError("Please specify either the name or id of the rule that starts the supply chain")
        self.Rally = Asset.Rally
        self.Asset = Asset
        self.id = id
        self.name = name
        if not id:
            try:
                self.id = self.Rally.apiCall("GET","/workflowRules?filter=name={}".format(urllib.parse.quote_plus(name)))["data"][0]["id"]
            except IndexError:
                raise ValueError("Could not find rule with name '{}'".format(name)) from None

    def run(self,initData={},endingPreset=None,timeout=15):
        baseWorkflow = self.Rally.apiCall("POST","/workflows",body={"data":{"type":"workflows", "attributes":{"initData":json.dumps(initData)}, "relationships": {"movie":{"data":{"id": self.Asset.id,"type": "movies",}},"workflowRule":{"data": {"id": self.id, "type": "workflowRules",}}}}})["data"]
        jobs = []
        processedJobs = []
        processedWorkflows = []
        delay = 5
        cancelAll = False
        start_time = time()
        while (start_time-time() < timeout) and not cancelAll:
            sleep(delay)
            baseWorkflow = self.Rally.apiCall("GET","/workflows/{}".format(baseWorkflow["id"]))["data"]
            workflows = [baseWorkflow]
            childWorkflows = [item for item in self.Rally.apiCall("GET","/workflows?filter=assetId={}".format(self.Asset.id),paginate=True)["data"] if item["relationships"]["baseWorkflow"]["data"]["id"] == baseWorkflow["id"] and item["id"] != baseWorkflow["id"]]
            workflows.extend(childWorkflows)
            if set([item["id"] for item in workflows]).issubset(set(processedWorkflows)):
                return jobs
            for workflow in workflows:
                if workflow["id"] not in processedWorkflows:
                    jobIds = [item["id"] for item in workflow["relationships"]["jobs"]["data"]]
                    for jobId in jobIds:
                        if jobId not in processedJobs:
                            job = self.Rally.apiCall("GET","/jobs/{}".format(jobId))
                            if job["data"]["attributes"]["status"] == "Complete":
                                processedJobs.append(jobId)
                                presetId = job["data"]["relationships"]["preset"]["data"]["id"]
                                jobPreset = self.Asset.preset(id=presetId)
                                jobPreset.getName()
                                newJob = jobPreset.job(id=jobId,attributes=job["data"]["attributes"])
                                jobs.append(newJob)
                                if endingPreset:
                                    if jobPreset.id == endingPreset.id:
                                        cancelAll = True
                    if workflow["attributes"]["state"] == "Complete":
                        processedWorkflows.append(workflow["id"])
        if cancelAll:
            baseWorkflow = self.Rally.apiCall("GET","/workflows/{}".format(baseWorkflow["id"]))["data"]
            workflows = [baseWorkflow]
            childWorkflows = [item for item in self.Rally.apiCall("GET","/workflows?filter=assetId={}".format(self.Asset.id),paginate=True)["data"] if item["relationships"]["baseWorkflow"]["data"]["id"] == baseWorkflow["id"] and item["id"] != baseWorkflow["id"]]
            workflows.extend(childWorkflows)
            for workflow in workflows:
                if workflow["id"] not in processedWorkflows:
                    jobIds = [item["id"] for item in workflow["relationships"]["jobs"]["data"]]
                    for jobId in jobIds:
                        if jobId not in processedJobs:
                            self.Rally.apiCall("PATCH","/jobs/{}".format(jobId),body={"data":{"type":"jobs", "attributes":{"state":"Cancelled"}}},fullResponse=True)
        return jobs


from actions_toolkit import core
from colorama import Fore
from colorama import Style
from colorama import init
import os

init()

class Test:

    def __init__(self):
        self.githubAction = bool(core.get_input('githubAction')) or os.environ.get('githubAction')
        self.tests = []
    
    def test(self,condition,testName,**kwargs):
        self.tests.append({
            "result":"pass" if condition else "fail",
            "name": testName,
            "kwargs": kwargs
        })
    
    def print(self):
        if any([item["result"] == "fail" for item in self.tests]) and self.githubAction:
            core.set_failed("All tests were not successful")
        print("---------------")
        for test in self.tests:
            print("\033[1m" + "Name: " + Style.RESET_ALL + Fore.CYAN + test["name"] + Style.RESET_ALL)
            print("\033[1m" + "Result: " + Style.RESET_ALL + (Fore.GREEN if test["result"] == "pass" else Fore.RED) + test["result"] + Style.RESET_ALL)
            for key in test["kwargs"]:
                print(Style.DIM + str(key) + " = " + str(test["kwargs"][key]))
            print(Style.RESET_ALL+"---------------")


import os
import requests
import json
from .Asset import Asset

class Rally:

    def __init__(self,configFromRallyTools=True,env="UAT",apiUrl=None,apiKey=None):
        self.env = env
        self.apiUrl = apiUrl
        self.apiKey = apiKey
        if configFromRallyTools:
            configPath = os.path.expanduser('~')+"/.rallyconfig"
            rallyConfig = open(configPath)
            rallyConfigJson = json.load(rallyConfig)
            apiConfig = rallyConfigJson["api"]
            rallyConfig.close()
            self.apiUrl = apiConfig[env]["url"]
            self.apiKey = apiConfig[env]["key"]
        elif not apiUrl or not apiKey:
            raise TypeError("Please specify both apiUrl and apiKey parameters")
    
    def apiCall(self,method,endpoint,body={},paginate=False,fullResponse=False,errors=True):
        headers={"Authorization":"Bearer {}".format(self.apiKey),"Content-Type":"application/json"}
        response = requests.request(method,headers=headers,url="{}{}".format(self.apiUrl,endpoint),data=json.dumps(body))
        if errors:
            response.raise_for_status()
        page = 2
        if not fullResponse or paginate:
            response = response.json()
        while paginate:
            if "?" in endpoint:
                url = "{}{}&page={}p10".format(self.apiUrl,endpoint,page)
            else:
                url = "{}{}?page={}p10".format(self.apiUrl,endpoint,page)
            results = requests.request(method,headers=headers,url=url,data=json.dumps(body))
            if errors:
                results.raise_for_status()
            response["data"].extend(results.json()["data"])
            if len(results.json()["data"]) < 10:
                paginate = False
            page+=1
        return response

    def asset(self,id=None,name=None):
        return Asset(self,id=id,name=name)